@@ -1,4 +1,4 @@
 #!/usr/bin/env perl
 use Dancer;
-use example;
+use example_FakeCGI;
 dance;
